# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals, division
from eight import *

from .lca import LCA
try:
    from bw2data import calculation_setups
except ImportError:
    calculation_setups = None
import numpy as np


class MultiLCA(object):
    """Wrapper class for performing LCA calculations with many functional units and LCIA methods.

    Needs to be passed a ``calculation_setup`` name.

    This class does not subclass the `LCA` class, and performs all calculations upon instantiation.

    Initialization creates `self.results`, which is a NumPy array of LCA scores, with rows of functional units and columns of LCIA methods. Ordering is the same as in the `calculation_setup`.

    """
    def __init__(self, cs_name):
        if not calculation_setups:
            raise ImportError
        assert cs_name in calculation_setups
        try:
            cs = calculation_setups[cs_name]
        except KeyError:
            raise ValueError(
                "{} is not a known `calculation_setup`.".format(cs_name)
            )
        self.activities = cs['inv']
        self.methods = cs['ia']
        self.lca = LCA(demand=self.all, method=self.methods[0])
        self.lca.lci(factorize=True)
        self.method_matrices = []
        self.results = np.zeros((len(self.activities), len(self.methods)))
        for method in self.methods:
            self.lca.method = method
            self.lca.load_lcia_data()
            self.method_matrices.append(self.lca.characterization_matrix)

        for row, activity in enumerate(self.activities):
            self.lca.redo_lci({activity[0]: float(activity[1])})
            for col, cf_matrix in enumerate(self.method_matrices):
                self.lca.characterization_matrix = cf_matrix
                self.lca.lcia_calculation()
                self.results[row, col] = self.lca.score

    @property
    def all(self):
        """Return a dictionary of all functional units"""
        return {k: float(v) for k, v in self.activities}



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals
from eight import *

from .utils import MAX_INT_32
import itertools
import numpy as np


def indexer(array_from, array_to, mapping):
    """
Chosen algorithm is faster than using rankdata, or simple iteration.
Modifies numpy arrays in place.

map(dict.get, data) seems faster, but doesn't work with missing values.

Setup code to try new approaches::

    from numpy import *
    from time import time
    a = random.random_integers(5000, size=100000)
    indices = sorted(list(set([int(x) for x in a])))
    mapping = dict(zip(indices, range(len(indices))))
    b = zeros(a.shape[0])

    """
    array_to[:] = np.array([mapping.get(x, MAX_INT_32) for x in array_from])


def dicter(array):
    """
Create dictionary from the sorted, unique values in a numpy array

Already more than fast enough. Setup code for other approaches::

        %%timeit import numpy as np; array = np.random.randint(0, 5000, size=50000)
        dict(zip(np.sort(np.unique(array)), itertools.count()))

    """
    return dict(zip(
        (int(x) for x in np.sort(np.unique(array))),
        itertools.count()
    ))



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals, division
from eight import *


class IndepentLCAMixin(object):
    """Mixin that allows `method`, etc. to be filepaths instead of DataStore object names.

    Removes dependency on `bw2data`."""
    def get_array_filepaths(self):
        """Pass through already correct values"""
        assert self._databases, "Must specify `databases` filepaths in independent LCA"
        return (
            self._databases,
            self.method,
            self.weighting,
            self.normalization,
        )



# -*- coding: utf-8 -*-

__all__ = [
    'DenseLCA',
    'GraphTraversal',
    'IndepentLCAMixin',
    'LCA',
    'LeastSquaresLCA',
    'MatrixBuilder',
    'MonteCarloLCA',
    'MultiMonteCarlo',
    'MultiLCA',
    'ParallelMonteCarlo',
    'ParameterVectorLCA',
    'TechnosphereBiosphereMatrixBuilder',
]

__version__ = (1, 2)

from .lca import LCA
from .dense_lca import DenseLCA
from .independent_lca import IndepentLCAMixin
from .least_squares import LeastSquaresLCA
from .monte_carlo import MonteCarloLCA, ParallelMonteCarlo, MultiMonteCarlo
from .multi_lca import MultiLCA
from .mc_vector import ParameterVectorLCA
from .graph_traversal import GraphTraversal
from .matrices import MatrixBuilder, TechnosphereBiosphereMatrixBuilder



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals
from eight import *

from .errors import MalformedFunctionalUnit
import itertools
import numpy as np
import os
try:
    import cPickle as pickle
except ImportError:
    import pickle


def load_arrays(paths):
    """Load the numpy arrays in list of filepaths ``paths``."""
    assert all(os.path.exists(fp) for fp in paths)
    return np.hstack([pickle.load(open(path, "rb")) for path in sorted(paths)])


def extract_uncertainty_fields(array):
    """Extract the core set of fields needed for uncertainty analysis from a parameter array"""
    fields = ["uncertainty_type", "amount", 'loc',
              'scale', 'shape', 'minimum', 'maximum', 'negative']
    return array[fields].copy()


try:
    from bw2data import (
        Database,
        databases,
        mapping,
        Method,
        methods,
        Normalization,
        normalizations,
        Weighting,
        weightings,
    )

    from bw2data.utils import TYPE_DICTIONARY, MAX_INT_32

    OBJECT_MAPPING = {
        'database': (Database, databases),
        'method': (Method, methods),
        'normalization': (Normalization, normalizations),
        'weighting': (Weighting, weightings),
    }

    def get_filepaths(name, kind):
        """Get filepath for datastore object `name` of kind `kind`"""
        if name is None:
            return None
        data_store, metadata = OBJECT_MAPPING[kind]
        assert name in metadata, "Can't find {} object {}".format(kind, name)
        return [data_store(name).filepath_processed()]

    def get_database_filepaths(functional_unit, database_list):
        """Get filepaths for all databases in supply chain of `functional_unit`"""
        dbs = set.union(*[Database(key[0]).find_graph_dependents() for key in functional_unit])
        return [Database(obj).filepath_processed() for obj in dbs]

    def clean_databases():
        databases.clean()
except ImportError:
    get_filepaths = get_database_filepaths = mapping = None

    def clean_databases():
        pass

    # Maximum value for unsigned integer stored in 4 bytes
    MAX_INT_32 = 4294967295
    TYPE_DICTIONARY = {
        "unknown": -1,
        "production": 0,
        "technosphere": 1,
        "biosphere": 2,
        "substitution": 3,
    }
